#################################################################################
#                                                                               #
#                       copyright (c) 2003 Ingeniweb SARL                       #
#                                                                               #
#################################################################################

"""PloneRSSSearchTool main class"""

from string import find, split, lstrip
from Products.CMFCore.utils import UniqueObject, _getAuthenticatedUser, _checkPermission
from Products.CMFCore.utils import getToolByName, _dtmldir
from OFS.Folder import Folder
from Globals import InitializeClass, DTMLFile, MessageDialog, ImageFile, PersistentMapping
from Acquisition import aq_base
from AccessControl.User import nobody
from AccessControl import ClassSecurityInfo
from Products.CMFCore.CMFCorePermissions import *
from Products.CMFCore.ActionProviderBase import ActionProviderBase


from urllib import quote, unquote, urlencode
from rssparser import parse, RSSParser

from ZPublisher import HTTPRequest
from base64 import decodestring, encodestring

try:
    import urllib2
    has_urllib2 = 1
except ImportError:
    import urllib
    has_urllib2 = 0

from urlparse import urlparse 

from global_symbols import *



class PloneRSSSearchTool(UniqueObject, Folder, ActionProviderBase):
    """
    PloneRSSSearchTool tool
    """
    id = 'portal_rsssearch'
    meta_type = 'PloneRSSSearchTool'
    _actions = []
    security = ClassSecurityInfo()
    #security.declareObjectPublic()
    #security.setDefaultAccess("allow")
    
    manage_options=(
        ({ 'label' : 'Overview'
           , 'action' : 'manage_overview'
           },) +
        ActionProviderBase.manage_options +
        Folder.manage_options
        )



    _properties=(
        {'id':'searchable_sites',
         'type': 'lines',
         'mode': 'w',
         },
         {'id':'_addCookie',
         'type': 'boolean',
         'mode': 'w',
         },
        )

    # Default values
    searchable_sites = []
    _addCookie = 1

    security.declarePublic('SearchPloneSite')
    def SearchPloneSite(self, plonesite_url, SearchableText):
        #url='%s/search_rss?SearchableText=%s' % (plonesite_url, quote(SearchableText))
        url='%s/search_rss?%s' % (plonesite_url, urlencode({'SearchableText':SearchableText}))
        if self._addCookie:
            results = self.useCookieForRequest(plonesite_url, SearchableText)[1]
        else:
            results = parse(url)[1]
        return results


    security.declarePublic('useCookieForRequest')
    def useCookieForRequest(self, plonesite_url, SearchableText):
        """
        Return the rss search result using the auth cookie.
        NB : plonesite_url must be a Plone2 site
             user must have same account (username and password) on the source
             and the target site.
        """
        urlHandler = None
        rssResponse = None

        value = self.getCookieAuthValue() #username:password
        try: userName, userPassword = value.split(':', 1)
        except AttributeError: userName=''; userPassword=''
        cookievalue = encodestring('%s:%s' % (userName, userPassword))
        cookieName = self.getCookieAuth()
        cookieValue = cookieName + '=' + cookievalue

        #theUrl = plonesite_url + '/search_rss?SearchableText=%s' % (SearchableText)
        theUrl='%s/search_rss?%s' % (plonesite_url, urlencode({'SearchableText':SearchableText}))

        if has_urllib2:                                       # Connection using urllib2
            req = urllib2.Request(theUrl)
            req.add_header('Cookie', cookieValue)
            urlHandler = urllib2.urlopen(req)
        else:
            opener = urllib.URLopener()
            opener.addHeader('Cookie', cookieValue)
            urlHandler = opener.open(theUrl)

        rssResponse = urlHandler.read()
        r = RSSParser()
        #parse the rss response body 
        r.feed(rssResponse)
        return r.channel, r.items


    security.declarePublic('getCookieAuth')
    def getCookieAuth(self,):
        """return the name of the auth cookie of CookieCrumbler"""
        return self.cookie_authentication.getProperty('auth_cookie')
    
    security.declarePublic('getCookieAuthValue')
    def getCookieAuthValue(self,):
        """
        return the value (decoded to string) of the auth cookie 
        (containing username:password)
        """
        myCookie = ''
        ac = self.getCookieAuth()
        REQUEST = self.REQUEST
        k = REQUEST['HTTP_COOKIE']
        cookies = {}
        HTTPRequest.parse_cookie(k, cookies)
        if cookies.has_key(ac):
            myCookie = cookies[ac]
        if myCookie:
            return str(decodestring(unquote(myCookie)))



    ## Some testing methods ##
    security.declarePublic('testUrl')
    def testUrl(self, url):
        if has_urllib2:                             # Connection using urllib2
            req = urllib2.Request(url)
            urlHandler = urllib2.urlopen(req)
        else:                                       # Connection using urllib
            opener = urllib.URLopener()
            urlHandler = opener.open(url)

        data = urlHandler.read()
        return data


InitializeClass(PloneRSSSearchTool)
